TopologyReady

Struct TopologyReady 

pub struct TopologyReady { /* private fields */ }
A handle for awaiting the readiness of a running topology.

Implementations

Source

impl TopologyReady

pub async fn wait(self) -> bool

Waits until the topology has registered its components and all of them have reported ready.

Returns true once the topology is fully ready, or false if the topology was torn down before it finished registering its components. The topology might be torn down before readiness is achieved if shutdown is requested while still waiting on an upstream dependency such as the environment provider.